/**
 * Architektur & Implementierung von DBMS, Aufgabenblatt 7
 *
 * Bitmap Indexing, Query Processing and WAH Compression
 *
 * File        : FieldType.h
 */

#ifndef FIELDTYPE_H
#define FIELDTYPE_H

#include <stdint.h>

/**
 * FieldType class holds the cardinality of the field which will be 
 * bitmap indexed. Additionally it provides a lookup method for field's values
 * to field's bitvector indices.
 */
template <typename T>
class FieldType {
  private:
    uint32_t mFieldCardinality;
    uint32_t mFieldOffset; /* offset of the field in the lineitem_t struct */

    inline T * extractValue(void * tuple) {
        return (T *)((char*)tuple + mFieldOffset);
    }

  public:
    FieldType(uint32_t fieldCardinality, uint32_t fieldOffset)
        : mFieldCardinality(fieldCardinality),
          mFieldOffset(fieldOffset)
      { }

    /** 
     * @return returns the cardinality of the field.
     */
    uint32_t getFieldCardinality() 
      {
          return mFieldCardinality;
      }

    /** 
     * Returns the bitvector index of the field for given value. 
     * For integers, just value modulo cardinality of the field.
     */
    inline int32_t lookup(void * tuple)
      {
          return (*extractValue(tuple) % mFieldCardinality);
      }

    /** 
     * Returns the bitvector index of the field for given value. 
     * For integers, just value modulo cardinality of the field.
     */
    inline int32_t lookupByValue(const T & value)
      {
          return (value % mFieldCardinality);
      }

};

/** 
 * Statically computed lookup table for shipmode field:
 * {FOB='0', 'MAIL'=1, 'SHIP'=3, 'REG AIR'=4, 'AIR'=5, 'RAIL'=6, 'TRUCK'=8}
 * 
 * @param tuple the next tuple containing shipmode's char[11] value
 * 
 * @return integer between 0 and 10
 */
template <> 
inline int32_t FieldType<char[11]>::lookup(void * tuple)
{
    int64_t val;
    val = *((int64_t *)*extractValue(tuple));

    return (val % 10);
}

template <> 
inline int32_t FieldType<char[11]>::lookupByValue(const char (&value)[11])
{
    int64_t val;
    val = *((int64_t *)value);

    return (val % 10);
}

/** 
 * Statically computed lookup table for returnflag:
 * {'A' = 0, 'N' = 1, 'R' = 2}
 * 
 * @param tuple the next tuple containing returnflag's char value
 * 
 * @return integer between 0 and 3
 */
template <> 
inline int32_t FieldType<char>::lookup(void * tuple)
{
    return (((int)*extractValue(tuple) - (int)'A') % 3);
}

template <> 
inline int32_t FieldType<char>::lookupByValue(const char & value)
{
    return (((int)value - (int)'A') % 3);
}

#endif /* FIELDTYPE_H */
